/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：信息安全性设计准则检查插件
 * 项目描述：用于检查源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */
package com.keyware.sonar.java;

import org.sonar.api.config.Configuration;
import org.sonar.api.config.PropertyDefinition;
import org.sonar.api.resources.AbstractLanguage;
import org.sonar.api.resources.Qualifiers;

import java.util.Arrays;
import java.util.List;

import static java.util.Arrays.asList;

public class ConfigurationFileLanguage extends AbstractLanguage {

    public static final String NAME = "Configuration";
    public static final String KEY = "cfg";
    public static final String FILE_SUFFIXES_KEY = "sonar.disposition.file.suffixes";
    public static final String FILE_SUFFIXES_DEFAULT_VALUE = ".properties,.ini,.conf,.xml,.yml,.json";

    private final Configuration config;

    public ConfigurationFileLanguage(Configuration config) {
        super(KEY, NAME);
        this.config = config;
    }

    @Override
    public String[] getFileSuffixes() {
        String[] suffixes = config.getStringArray(FILE_SUFFIXES_KEY);

        if (suffixes == null || suffixes.length == 0) {
            return FILE_SUFFIXES_DEFAULT_VALUE.split(",");
        }

        // 修剪空格和小写后缀
        return Arrays.stream(suffixes)
                .map(String::trim)
                .map(String::toLowerCase)
                .toArray(String[]::new);
    }

    public static List<PropertyDefinition> getProperties() {
        return asList(
                PropertyDefinition.builder(FILE_SUFFIXES_KEY)
                        .multiValues(true)
                        .defaultValue(FILE_SUFFIXES_DEFAULT_VALUE)
                        .category("FileType")
                        .name("File Suffixes")
                        .description("List of suffixes for files to analyze.")
                        .onQualifiers(Qualifiers.PROJECT)
                        .build()
        );
    }
}
