/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：信息安全性设计准则检查插件
 * 项目描述：用于检查源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */
package com.keyware.sonar.java.rules.checkers;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.sonar.api.batch.fs.InputFile;
import org.sonar.api.batch.sensor.SensorContext;
import org.sonar.api.batch.sensor.issue.NewIssue;
import org.sonar.api.rule.RuleKey;
import org.sonar.check.Rule;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.yaml.snakeyaml.Yaml;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import java.io.*;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.Scanner;


/**
 * TODO ConfigurationFileChecker
 *
 * @author WuHaoYang
 * @date 2024/1/22
 */
@Rule(key = "ConfigurationFileChecker")
public class ConfigurationFileChecker {


    public void execute(SensorContext context, InputFile inputFile, RuleKey ruleKey){
        //文件名称
        String filename = inputFile.filename();
        System.out.println("[ConfigurationFileChecker]>>>>>" + filename);

        //校验文件后缀
        if (filename.endsWith(".properties")) {
            try {
                File file = new File(inputFile.absolutePath());
                System.out.println("---------------properties文件路径----------------"+file);
                try (Scanner scanner = new Scanner(file)) {
                    int lineNum = 1;
                    while (scanner.hasNextLine()) {
                        String line = scanner.nextLine();
                        if (line.contains("password")) {
                            System.out.println(line);
                            NewIssue newIssue = context.newIssue();
                            newIssue
                                    .forRule(ruleKey)
                                    .at(newIssue.newLocation()
                                    .on(inputFile)
                                    .at(inputFile.selectLine(lineNum)))
                                    .save();
                            break;
                        }
                        lineNum ++;
                    }
                }
            } catch (FileNotFoundException e) {
                System.out.println("文件未找到: " + e.getMessage());
            }
        }


        if (filename.endsWith(".ini")){
            // 获取当前输入文件的绝对路径
            File file1 = inputFile.file();
            File absoluteFile = file1.getAbsoluteFile();

            // 构建目录路径
            File folder = new File(String.valueOf(absoluteFile)).getParentFile();
            System.out.println("---------------ini文件路径----------------"+folder);
            File[] listOfFiles = folder.listFiles();

            int lineNum = 1;
            for (File file : listOfFiles) {
                if (file.isFile() && file.getName().endsWith(".ini")) {
                    Properties properties = new Properties();

                    try (FileInputStream fileInput = new FileInputStream(file)) {
                        properties.load(fileInput);
                        String password = properties.getProperty("password");
                        System.out.println("password=" + password);
                        NewIssue newIssue = context.newIssue();
                        newIssue
                                .forRule(ruleKey)
                                .at(newIssue.newLocation()
                                .on(inputFile)
                                .at(inputFile.selectLine(lineNum)))
                                .save();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
                lineNum++;
            }
        }



        if (filename.endsWith(".conf")){
            // 获取当前输入文件的绝对路径
            File file1 = inputFile.file();
            File absoluteFile = file1.getAbsoluteFile();

            // 构建目录路径
            File folder = new File(String.valueOf(absoluteFile)).getParentFile();

            System.out.println("---------------conf文件路径----------------"+folder);

            File[] listOfFiles = folder.listFiles((dir, name) -> name.endsWith(".conf"));

            int lineNum = 1;
            for (File file : listOfFiles) {
                if (file.isFile()) {
                    Properties prop = new Properties();
                    InputStream input = null;

                    try {
                        input = new FileInputStream(file);
                        prop.load(input);

                        if (prop.containsKey("password")) {
                            System.out.println("password="+ prop.getProperty("password"));
                            NewIssue newIssue = context.newIssue();
                            newIssue
                                    .forRule(ruleKey)
                                    .at(newIssue.newLocation()
                                            .on(inputFile)
                                            .at(inputFile.selectLine(lineNum)))
                                    .save();
                        }

                    } catch (IOException ex) {
                        ex.printStackTrace();
                    } finally {
                        if (input != null) {
                            try {
                                input.close();
                            } catch (IOException e) {
                                e.printStackTrace();
                            }
                        }
                    }
                }
                lineNum++;
            }
        }



        if (filename.endsWith(".xml")){
            // 获取当前输入文件的绝对路径
            File file1 = inputFile.file();
            File absoluteFile = file1.getAbsoluteFile();

            // 构建目录路径
            File dir = new File(String.valueOf(absoluteFile)).getParentFile();
            System.out.println("---------------xml文件路径----------------"+dir);
            FilenameFilter filter = new FilenameFilter() {
                public boolean accept(File dir, String name) {
                    return name.endsWith(".xml");
                }
            };

            String[] children = dir.list(filter);
            if (children == null) {
                System.out.println("目录不存在或不是目录");
            } else {
                for (int i = 0; i < children.length; i++) {
                    String filename1 = children[i];
                    File xmlFile = new File(dir, filename1);
                    processXML(xmlFile);
                }
            }
        }


        if (filename.endsWith(".json")){
            try {
                // 获取当前输入文件的绝对路径
                File file1 = inputFile.file();
                File absoluteFile = file1.getAbsoluteFile();

                // 构建目录路径
                File folder = new File(String.valueOf(absoluteFile)).getParentFile();
                System.out.println("---------------json文件路径----------------"+folder);
                File[] listOfFiles = folder.listFiles();

                if (listOfFiles != null) {
                    ObjectMapper mapper = new ObjectMapper();
                    for (File file : listOfFiles) {
                        if (file.isFile() && file.getName().endsWith(".json")) {
                            JsonNode rootNode = mapper.readTree(file);
                            extractPassword(rootNode);
                        }
                    }
                }

            } catch (IOException e) {
                e.printStackTrace();
            }
        }


        if (filename.endsWith(".yml")){
            // 获取当前输入文件的绝对路径
            File file1 = inputFile.file();
            File absoluteFile = file1.getAbsoluteFile();

            // 构建目录路径
            File dir = new File(String.valueOf(absoluteFile)).getParentFile();
            System.out.println("---------------yml文件路径----------------"+dir);
            Yaml yaml = new Yaml();
            for (File file : dir.listFiles()) {
                if (file.isFile() && file.getName().endsWith(".yml")) {
                    try (FileInputStream fis = new FileInputStream(file)) {
                        Map<String, Object> obj = yaml.load(fis);
                        if (obj != null){
                            String password = searchPassword(obj);
                            if (password != null) {
                                System.out.println("password="+password);
                            }
                        }
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }
        }

    }
    public static void processXML(File xmlFile) {
        try {
            DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
            Document doc = dBuilder.parse(xmlFile);
            doc.getDocumentElement().normalize();

            NodeList nList = doc.getElementsByTagName("password");

            for (int i = 0; i < nList.getLength(); i++) {
                Node nNode = nList.item(i);
                if (nNode.getNodeType() == Node.ELEMENT_NODE) {
                    Element eElement = (Element) nNode;
                    System.out.println("Password="+ eElement.getTextContent());
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    public static void extractPassword(JsonNode node) {
        Iterator<String> fieldNames = node.fieldNames();
        while (fieldNames.hasNext()) {
            String fieldName = fieldNames.next();
            if (fieldName.equals("password")) {
                System.out.println("Password= " + node.get(fieldName).asText());
            }
            if (node.get(fieldName).isContainerNode()) {
                extractPassword(node.get(fieldName));
            }
        }
    }



    private static String searchPassword(Map<String, Object> map) {
        for (String key : map.keySet()) {
            if ("password".equals(key) && map.get(key) instanceof String) {
                return (String) map.get(key);
            } else if (map.get(key) instanceof Map) {
                String password = searchPassword((Map<String, Object>) map.get(key));
                if (password != null) {
                    return password;
                }
            }
        }
        return null;
    }
}
